/* wptKeyPropsDlg.cpp - WinPT key properties dialog
 *	Copyright (C) 2000, 2001, 2002, 2003, 2005, 2006 Timo Schulz
 *
 * This file is part of WinPT.
 *
 * WinPT is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * WinPT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with WinPT; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <windows.h>

#include "resource.h"
#include "wptErrors.h"
#include "wptGPG.h"
#include "wptCommonCtl.h"
#include "wptContext.h" /* for passphrase_s */
#include "wptNLS.h"
#include "wptDlgs.h" 
#include "wptTypes.h"
#include "wptKeylist.h"
#include "wptW32API.h"
#include "wptVersion.h"
#include "wptKeyEdit.h"


/* Check that the key is not expired or revoked. */
static int
do_check_key (gpgme_key_t key)
{
    int okay = 0;
    okay = key->expired;
    if (!okay)
	okay = key->revoked;
    return okay;
}


/* Convert a trust integer into a string representation. */
static const char*
ownertrust_to_string (int val, bool is_keypair)
{
    const char *inf;
    int id = val;

    switch (id) {
    case 1: inf = _("Don't know");         break;
    case 2: inf = _("I do NOT trust");     break;
    case 3: inf = _("I trust marginally"); break;
    case 4: inf = _("I trust fully");      break;
    case 5:
    case 6: 
	if (is_keypair)
	    inf = _("I trust ultimately (implicit)");
	else
	    inf = _("I trust ultimately"); break;
    default:inf = _("Unknown");            break;
    }
    
    return inf;
}


/* Generate a unique temp name for the photo which
   depends on the dialog handle and return it. */
static const char*
get_photo_tmpname (HWND dlg)
{
    static char buf[MAX_PATH+128+1];
    char name[64];

    _snprintf (name, sizeof (name)-1, "winpt_photo_%08lX.tmp", (DWORD)dlg);
    get_temp_name (buf, DIM (buf), name);
    return buf;
}


static void
draw_nophoto_img (HWND dlg)
{
    /*..
    n = DrawText (hdc, "No Photo-ID", -1, &r, DT_LEFT);
    ..*/
}


/* Load the photo from the key @key */
static int
keyprops_load_photo (HWND dlg, gpgme_key_t key, gpgme_validity_t *r_valid)
{
    winpt_key_s k;
    FILE *f;
    const BYTE *img;
    DWORD imglen = 0;
    int pos=0;

    winpt_get_pubkey (key->subkeys->keyid, &k);
    img = k.ext->attrib.d;
    imglen = k.ext->attrib.len;
    if (!k.ext->attrib.validity)
	get_uat_validity (key->subkeys->keyid, &k.ext->attrib.validity);
    *r_valid = k.ext->attrib.validity;

    if (!img || !imglen) {
	draw_nophoto_img (dlg);
	return -1;
    }

    f = fopen (get_photo_tmpname (dlg), "wb");
    if (f) {
	//for (pos = 0; img[pos] != 0x10; pos++)
	//	;
	pos += 16;
	fwrite (img + pos, 1, imglen - pos, f);
	fclose (f);
    }
    return 0;
}


/* Display the photo in the image control in the dialog @dlg. */
static int
keyprops_show_photo (HWND dlg)
{
    RECT r;	
    POINT p;
    HWND h;

    h = GetDlgItem (dlg, IDC_KEYPROPS_IMG);
    GetWindowRect (h, &r);
    p.x = r.left + 5;
    p.y = r.top - 2;
    memset (&p, 0, sizeof (p));
    PTD_jpg_show (h, &p, get_photo_tmpname (dlg));
    
    return 0;
}


/* Return string representation of the key validity. @key. */
static const char*
get_validity (gpgme_key_t key)
{
    int val;
    val = key->expired;
    if (val)
	return _("Expired");	
    val = key->revoked;
    if (val)
	return _("Revoked");
    val = key->disabled;
    if (val)
	return _("Disabled");
    return get_key_trust2 (NULL, key->uids->validity, 0, 0);
}


/* Return the preferred sym. algorithm from @key as a string. */
static const char*
get_pref_cipher (winpt_key_t k)
{
    if (k->is_v3)
	return "IDEA";
    if (!k->ext->sym_prefs)
	return "3DES";
    switch (*k->ext->sym_prefs) {
    case 1: return "IDEA";
    case 2: return "3DES";
    case 3: return "CAST5";
    case 4: return "Blowfish";
    case 7:
    case 8:
    case 9: return "AES";
    case 10:return "Twofish";
    }
    return "Unknown";
}


/* Return true if the key has designated revokers. */
static bool
check_for_desig_rev (gpgme_key_t key)
{
    winpt_key_s k;
    memset (&k, 0, sizeof (k));
    if (!winpt_get_pubkey (key->subkeys->keyid, &k))
	return k.ext->gloflags.has_desig_rev? true : false;
    return false;
}


/* Print information (name) of the smart card. */
static const char*
get_card_type (winpt_key_t k)
{    
    static char buf[64];

    if (!k->ext || !k->ext->card_type)
	return "";
    _snprintf (buf, sizeof (buf)-1, _("Card-Type: %s\r\n"), k->ext->card_type);
    return buf;
}


/* Display the key information for key @k.
   Return value: gpgme key on success. */
static void
display_key_info (HWND dlg, winpt_key_t k, gpgme_key_t *r_key)
{
    gpgme_key_t key;
    struct winpt_key_s pk, sk;
    char info[512];
    const char *inf;
    DWORD created, expires;

    memset (&pk, 0, sizeof (pk));
    if (winpt_get_pubkey (k->keyid, &pk))
	BUG (0);
    gpg_keycache_update_attr (pk.ext, KC_ATTR_PREFSYM, 0);
    memset (&sk, 0, sizeof (sk));	
    if (k->key_pair && !winpt_get_seckey (k->keyid, &sk))
	k->is_protected = sk.is_protected;
    key = pk.ext->key;
    created = key->subkeys->timestamp;
    expires = key->subkeys->expires;
    _snprintf (info, DIM (info)-1,
               _("Type: %s\r\n"
               "Key ID: %s\r\n"
               "Algorithm: %s\r\n"
               "Size: %s bits\r\n"
               "Created: %s\r\n"
	       "Expires: %s\r\n"
               "Validity: %s\r\n"
	       "Cipher: %s\r\n"
	       "%s\r\n"),
               get_key_type (key),
               k->keyid,
               get_key_algo (key, 0),
               get_key_size (key, 0),
               get_key_created (created),
	       get_key_expire_date (expires),
               get_validity (key), 
	       get_pref_cipher (&pk),
	       get_card_type (&sk));

    SetDlgItemText (dlg, IDC_KEYPROPS_INFO, info);
    SetDlgItemText (dlg, IDC_KEYPROPS_FPR, get_key_fpr (key));  
    inf = ownertrust_to_string (key->owner_trust, k->key_pair);
    SetDlgItemText (dlg, IDC_KEYPROPS_OT, inf);

    *r_key = key;
}


/* Dialog box procedure to show the key properties. */
BOOL CALLBACK
keyprops_dlg_proc (HWND dlg, UINT msg, WPARAM wparam, LPARAM lparam)
{
    static winpt_key_t k;
    static gpgme_key_t key;
    gpgme_validity_t valid;
    const char *inf;
    int rc;
    
    /* XXX: static variable (k) prevent that the dialog can 
	    be opened twice. */
    switch (msg) {
    case WM_INITDIALOG:
        if (!lparam)
            dlg_fatal_error (dlg, "Could not get dialog param!");
        k = (winpt_key_t)lparam;
        SetWindowText (dlg, _("Key Properties"));
	SetDlgItemText (dlg, IDC_KEYPROPS_OT_CHANGE, _("&Change"));
	SetDlgItemText (dlg, IDC_KEYPROPS_REVOKERS, _("&Revokers"));
	SetDlgItemText (dlg, IDC_KEYPROPS_CHANGE_PWD, _("Change &Password"));
	SetDlgItemText (dlg, IDC_KEYPROPS_OTINF, _("Ownertrust"));
	
	display_key_info (dlg, k, &key);
	if (!keyprops_load_photo (dlg, key, &valid)) {
	    k->has_photo = 1;	
	    if (valid < GPGME_VALIDITY_MARGINAL)
		SetDlgItemText (dlg, IDC_KEYPROPS_IMGINF, _("Photo-ID not validated."));
	}
        if (k->key_pair)
            EnableWindow (GetDlgItem (dlg, IDC_KEYPROPS_CHANGE_PWD), TRUE);
	if (check_for_desig_rev (key))
	    EnableWindow (GetDlgItem (dlg, IDC_KEYPROPS_REVOKERS), TRUE);
	if (key->revoked || key->expired)
	    EnableWindow (GetDlgItem (dlg, IDC_KEYPROPS_OT_CHANGE), FALSE);
	center_window (dlg, NULL);
        SetForegroundWindow (dlg);	
        return TRUE;

    case WM_DESTROY:
	remove (get_photo_tmpname (dlg));
	break;
      
    case WM_PAINT:
	if (k->has_photo)
	    keyprops_show_photo (dlg);
	break;

    case WM_SYSCOMMAND:
        if (LOWORD (wparam) == SC_CLOSE)
            EndDialog (dlg, FALSE);
        return FALSE;
        
    case WM_COMMAND:
        switch (LOWORD (wparam)) {
        case IDOK:
            EndDialog (dlg, TRUE);
            return TRUE;

	case IDCANCEL:
	    EndDialog (dlg, FALSE);
	    return TRUE;
            
        case IDC_KEYPROPS_OT_CHANGE:
	    if (do_check_key (key)) {
		msg_box (dlg, _("The status of this key is 'revoked' or 'expired'.\n"
				"You cannot change the ownertrust of such keys."),
				_("WinPT Warning"), MB_ERR);
		return TRUE;
	    }
	    if (!k->key_pair && key->uids->validity < 3) {
		rc = msg_box (dlg, _("This is a non-valid key.\n"
				     "Modifying the ownertrust has no effect on such keys.\n\n"
			             "Do you really want to continue?"), 
				     _("WinPT Warning"), MB_ICONWARNING|MB_YESNO);
		if (rc == IDNO)
		    return TRUE;
	    }
            rc = dialog_box_param (glob_hinst, 
				   (LPCSTR)IDD_WINPT_KEYEDIT_OWNERTRUST,
				    dlg, keyedit_ownertrust_dlg_proc,
				   (LPARAM)k, _("Change Ownertrust"),
				    IDS_WINPT_KEYEDIT_OWNERTRUST);
	    if (rc == FALSE) /* Cancel */
		return TRUE;

	    inf = ownertrust_to_string (k->callback.new_val, k->key_pair);
            SetDlgItemText (dlg, IDC_KEYPROPS_OT, inf);
            msg_box (dlg, _("Ownertrust successfully changed."), 
		     _("GnuPG Status"), MB_OK);
	    k->update = 1;
            return TRUE;
            
        case IDC_KEYPROPS_CHANGE_PWD:
            keyedit_change_passwd (k, dlg);	    
            return TRUE;

	case IDC_KEYPROPS_REVOKERS:
	    k->update = dialog_box_param (
		glob_hinst, (LPCTSTR)IDD_WINPT_KEYREVOKERS, dlg,
		key_revokers_dlg_proc, (LPARAM)key, 
		_("Key Revokers"), IDS_WINPT_KEY_REVOKERS);
	    break;
        }
    }
    
    return FALSE;
}
